//=============================================================================
// Filename: Example_xlcd2.c
//-----------------------------------------------------------------------------
// Compiled using MPLAB-C18 v3.34 student edition
//=============================================================================
// Company	: Cytron Technologies, Malaysia
// Revision	: 1.00
// Date		: 28 Dec 2009
//=============================================================================
#include <p18f4520.h>
#include "xlcd.h"
#include "delays.h"


//=============================================================================
//	Configuration Bits
//=============================================================================
#pragma	config OSC = HS				// HS oscillator
#pragma	config FCMEN = OFF			// Fail-Safe Clock Monitor disabled
#pragma	config IESO = OFF			// Oscillator Switchover mode disabled
#pragma	config PWRT = OFF			// PWRT disabled
#pragma	config BOREN = OFF			// Brown-out Reset disabled in hardware and software 
#pragma	config WDT = OFF			// WDT disabled (control is placed on the SWDTEN bit)
#pragma	config MCLRE = ON			// MCLR pin enabled; RE3 input pin disabled 
#pragma	config PBADEN = OFF			// PORTB<4:0> pins are configured as digital I/O on Reset 
#pragma	config CCP2MX = PORTC		// CCP2 input/output is multiplexed with RC1 
#pragma	config LVP = OFF			// Single-Supply ICSP disabled 
#pragma	config XINST = OFF			// Extended Instruction Set 
//=============================================================================
//	Define Pins
//=============================================================================
#define	led1		LATBbits.LATB6		//active High
#define	led2		LATBbits.LATB7

#define sw1			PORTBbits.RB0
#define sw2			PORTBbits.RB1


//=============================================================================
//	Function Prototypes
//=============================================================================
void Delay_1msX (unsigned int miliseconds);
void Delay_100msX (unsigned int msec);


//=============================================================================
//	Global Variables
//=============================================================================
unsigned int i, j, t;


//=============================================================================
//	Main Program
//=============================================================================
void main ()
{
	// Space    = full white box, no bar  
	// <others> = graphics defined in the array.  
	// 255      = full black box, full bar  
	const char display_bar[9] = {' ',0,1,2,3,4,5,6,255};
	unsigned char cursor=20, level=0;

	//set I/O input output
	TRISB = 0b00000011;					//Configure PORTB I/O direction
	TRISD = 0b00000000;					//Configure PORTD I/O direction
	PORTB = 0;
	PORTD = 0;

	//-------------------------------------------------------------------------
	// Configure External LCD
  	//-------------------------------------------------------------------------
	OpenXLCD( EIGHT_BIT & LINES_5X7 );
	ClearXLCD();		             	//Clear display
	
	//-------------------------------------------------------------------------
	// User-defined Graphics
	//-------------------------------------------------------------------------
	SetCGRamAddr(0x40);					//Goto CGRAM address #1

	// Bar Graph 1
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);

	// Bar Graph 2
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Bar Graph 3
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Bar Graph 4
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Bar Graph 5
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Bar Graph 6
	putcXLCD(0b00000);
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Bar Graph 7
	putcXLCD(0b00000);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);
	putcXLCD(0b11111);

	// Non-standard 'A'
	putcXLCD(0b00100);
	putcXLCD(0b01010);
	putcXLCD(0b10001);
	putcXLCD(0b10001);
	putcXLCD(0b11111);
	putcXLCD(0b10001);
	putcXLCD(0b10001);
	putcXLCD(0b10001);

	ClearXLCD();		             			//Clear display
	led1=1;										//Turn on led1	

	//-------------------------------------------------------------------------
	// Display External LCD
	//-------------------------------------------------------------------------
	SetCurXLCD(0);
	
	for(i=1;i<7;i++)
	{
		putcXLCD(display_bar[i]);				//Display all user-defined graphics at line 1.
	}
	putcXLCD(7);								//Display user-defined non-standard 'A'
	putcXLCD('A');								//Display standard 'A' using putcXLCD
	putrsXLCD("A");								//Display standard 'A' using putrsXLCD

	while(1)
	{
		if(!sw1)								//Test sw1 to change bar level
		{
			while(!sw1) led1=1;					//Debounce sw1, Turn on led1
			led1=0;								//Turn off led1
			Delay_100msX(2);					//Delay 200ms
			level++;							//Increase bar level
			if(level>8) level=0;				//If level exceeds 8, set as 0
			SetCurXLCD(cursor);					//Set cursor to remain at current position
			putcXLCD(display_bar[level]);		//Display bar level
		}
		else if(!sw2)							//Test sw2 to change cursor position
		{
			while(!sw2) led2=1;					//Debounce sw2, Turn on led2
			led2=0;								//Turn off led2
			Delay_100msX(1);					//Delay 100ms
			cursor++;							//Increase lcd cursor position
			if(cursor>35) cursor=20;			//If cursor position exceeds 35, set as 20
			SetCurXLCD(cursor);					//Set cursor position
			level=0;							//Clear level
		}
	}
}//End of main



//=============================================================================
//	Subroutines
//=============================================================================

/********************************************************************
*       Function Name:  Delay_1msX                               	*
*       Return Value:   void                                        *
*       Parameters:     miliseconds: amount of delay				*
*       Description:    This routine generates various delays 		*
*						needed by xlcd functions.					*
*						For delay of 1ms (18F4550 running at 48MHz)	*
*						Cycles = (TimeDelay * Fosc) / 4				*
*						Cycles = (1ms * 48MHz) / 4					*
*						Cycles = 12,000								*
*						Since call of function also takes some 		*
*						instruction cycles, the exact value to get	*
*						1ms delay is less than 12,000.				*
********************************************************************/
void Delay_1msX (unsigned int miliseconds)
{
	t=0;
	while(t<miliseconds)
	{
		Delay1KTCYx(11);
		Delay10TCYx(96);
		Nop();
		Nop();
		Nop();
		Nop();
		Nop();
		t++;
	}
}//End of Delay_1msX


/********************************************************************
*       Function Name:  Delay_100msX                               	*
*       Return Value:   void                                        *
*       Parameters:     miliseconds: amount of delay				*
*       Description:    This routine generates various delays 		*
*						needed by xlcd functions.					*
*						For delay of 100ms							*
*						(18F4550 running at 48MHz)					*
*						Cycles = (TimeDelay * Fosc) / 4				*
*						Cycles = (100ms * 48MHz) / 4				*
*						Cycles = 1,200,000							*
*						Since call of function also takes some 		*
*						instruction cycles, the exact value to get	*
*						1ms delay is less than 1,200,000.			*
********************************************************************/
void Delay_100msX (unsigned int msec)
{
	t=0;
	while(t<msec)
	{
		Delay10KTCYx(119);
		Delay1KTCYx(9);
		Delay10TCYx(96);
		t++;
	}
}//End of Delay_100msX
